## command line options
set options(-host) localhost
set ::SCOE GAL-TTC
set options(-loglevel) T
array set options $argv

set ::EGSE_simulator 1

## packages
package require BWidget
package require Itcl
package require Iwidgets
package require Itk
catch {namespace import itcl::*}
catch {namespace import itk::*}

proc parse {args} {}
proc getclock {args} {return [clock seconds]}

## the global directories
set sfile [info script]
if {[file pathtype $sfile] != "absolute"} {
    set sfile [file join [pwd] $sfile]
}
set ::SCOE_PATH [file dir [file dir $sfile]]

## --------------------------------------------------------------------
## Platform specific
## --------------------------------------------------------------------
set sfile [info script]
if {[file pathtype $sfile] != "absolute"} {
    set sfile [file join [pwd] $sfile]
}
set ::SCOE_PATH [file dir [file dir $sfile]]


global env APP_DIR
if {[info exists env(${::SCOE}_SCOE_PATH)]} {
    set APP_DIR $env(${::SCOE}_SCOE_PATH)
} else {
    set APP_DIR [file dir [file dir $sfile]]
}

## the global directories
foreach dir {classes database gui hardware network nobjects util widgets} {
    set fullDir [file join $::SCOE_PATH tcl $dir]
    if {[file isdir $fullDir]} {
        lappend ::auto_path $fullDir
        ## generate global variable
        set "::[string toupper $dir]_GPATH" $fullDir
    }
}

## check the most common paths
foreach dir {config pictures} {
    set found 0
    foreach fullDir [list [file join [file normalize .] $dir] \
                         [file join [file dirname [info nameofexecutable]] $dir] \
                         [file join $::SCOE_PATH $dir] \
                         [file join $::SCOE_PATH tcl scoe $::SCOE $dir]] {
        if {[file isdir $fullDir]} {
            set "::[string toupper $dir]_GPATH" $fullDir
            puts "Common $dir path is '$fullDir'"
            set found 1
            break
        }
    }
    if {!$found} {
        error "No common $dir path found for $::SCOE"
    }
}

## put scoe path first
lappend ::auto_path $::SCOE_PATH/tcl/scoe/$::SCOE $::CONFIG_GPATH

## then scoe-specific directories
set ::CONFIG_PATH -1
set ::PICTURES_PATH -1
foreach dir {config pictures classes database gui hardware network nobjects util widgets cmdif testseq} {
    set fullDir [file join $::SCOE_PATH tcl scoe $::SCOE $dir]
    if {[file isdir $fullDir]} {
        lappend ::auto_path $fullDir
        ### generate global variable
        set "::[string toupper $dir]_PATH" $fullDir
    }
}
if {$::CONFIG_PATH == -1} {set ::CONFIG_PATH $::CONFIG_GPATH}
if {$::PICTURES_PATH == -1} {set ::PICTURES_PATH $::PICTURES_GPATH}

InitConfig
## nobject interface
set ::SCOE_CONFIG(scoe.NObIfObj) [NObHandler "#auto"]
#$::SCOE_CONFIG(scoe.NObIfObj) configure -systConfigPath $::CONFIG_PATH
## Create object Scheduler
set ::SCOE_CONFIG(scoe.Scheduler) [TimerSupport "#auto"]

## standard objects
$::SCOE_CONFIG(scoe.NObIfObj) addnob -class CMDFIN_hktm
$::SCOE_CONFIG(scoe.NObIfObj) addnob -class SAFETY_hktm
$::SCOE_CONFIG(scoe.NObIfObj) addnob -class SCOE_hktm
$::SCOE_CONFIG(scoe.NObIfObj) addnob -class RESULT_hktm
$::SCOE_CONFIG(scoe.NObIfObj) create

## others
set ::simulation server

## system states
set ::SCOE_activity "IDLE"
set ::SCOE_state "LOCAL"
set ::SCOE_uplink "UNKNOWN"
set ::SCOE_selftest "PASSED"
set ::Selftest_state "passed"
set ::EGSE_connection "DOWN"
set ::SCOE_acknak "NONE"
set ::SCOE_hktm "NONE"

## --------------------------------------------------------------------
## GUI
## --------------------------------------------------------------------
## main title of top window
wm geometry . 500x200+10+10
wm title . "CCS simulator for $::SCOE"
wm protocol . WM_DELETE_WINDOW "exit"

## menu
menu .menus
menu .menus.showHKTM
menu .menus.options

.menus add cascade -label "Options" -menu .menus.options
.menus add cascade -label "HKTMs" -menu .menus.showHKTM

. configure -menu .menus

if {$::tcl_platform(platform) == {windows}} {
    .menus.options add command -label "Console" -command {console show}
} else {
    .menus.options add command -label "Console" -command {exec tkcon -eval {tkcon attach CCS_sim.tcl} &}
}

## logging
InitLogging

## Status frame
frame .statusFrame
pack .statusFrame -fill x

gen_statefield .statusFrame.connection "Socket:" ::EGSE_connection {
    { "DOWN"        orange  "DOWN" }
    { "CONNECTED"   green   "CONNECTED" }
}

## for our simulator make it a different window
foreach nobj [$::SCOE_CONFIG(scoe.NObIfObj) getnobOption -type ghktmnobject] {
    set obj [$::SCOE_CONFIG(scoe.NObIfObj) getnob $nobj -object]
    $obj confvar -rwtype hktm
}

proc show {} {
    showNObjectWindow CMDFIN
    showNObjectWindow SAFETY
    showNObjectWindow SCOE
    showNObjectWindow RESULT
}

proc crash {} {
    foreach w [find object -isa GenericWindow] {
        destroy $w
    }
    destroy .
}

console show