#History        : $Log: guiutil.tcl,v $
#History        : Revision 1.12  2007/02/12 16:22:00  perschak.t
#History        : inverting enabled_states to disabled_states
#History        :
#History        : Revision 1.11  2007/02/06 15:02:36  perschak.t
#History        : beautified the logging
#History        :
#History        : Revision 1.10  2007/02/01 15:53:50  perschak.t
#History        : renamed confvar, getvar and addvar to confcomp, getcomp and addcomp
#History        :
#History        : Revision 1.9  2006/12/13 13:57:51  papp.v
#History        : Added proc showToplevel.
#History        :

proc operatorClock {text args} {
    ## define range for options
    set optionsRange(-autoClock) {-values {yes no y n 1 0}}
    set optionsRange(-init) {-values {yes no y n 1 0}}
    set optionsRange(-steps) {-minmax 1:+ -values {{}}}
    ## define allowed options
    set options(-autoClock) yes
    set options(-title) {}
    set options(-steps) {}
    set options(-background) "clock_default"
    set options(-init) yes
    ## set allowed
    set optionsAllowed [array names options]

    ## read args so we can set defaults for the type
    array set options $args

    ## check for unallowed/range options
    checkCompAllowed options $optionsAllowed
    checkCompRange options optionsRange

    if {![info exists ::SCOE_CONFIG(windows.OpClock)] || ($::SCOE_CONFIG(windows.OpClock) == {})} {
        set ::SCOE_CONFIG(windows.OpClock) [OperatorClockWindow ".#auto"]
        $::SCOE_CONFIG(windows.OpClock) configure -symbolPath [file join $::PICTURES_GPATH clock]
    }
    if {[valbool $options(-init)]} {$::SCOE_CONFIG(windows.OpClock) init}
    $::SCOE_CONFIG(windows.OpClock) configure -symbolFile $options(-background) -text $text
    $::SCOE_CONFIG(windows.OpClock) configure -autoClock $options(-autoClock)

    if {$options(-title) != {}} {
        $::SCOE_CONFIG(windows.OpClock) configure -title $options(-title)
    }
    if {$options(-steps) != {}} {
        $::SCOE_CONFIG(windows.OpClock) configure -enableRFeedback y
        $::SCOE_CONFIG(windows.OpClock) fbReset
        $::SCOE_CONFIG(windows.OpClock) fbSteps $options(-steps)
    } else {
        $::SCOE_CONFIG(windows.OpClock) configure -enableRFeedback n
    }

    ## show window
    $::SCOE_CONFIG(windows.OpClock) configure -show y
}

proc operatorOk {text args} {
    ## define range for options
    set optionsRange(-init) {-values {yes no y n 1 0}}
    set optionsRange(-type) {-values {note info question warning error clockstep}}
    set optionsRange(-foreLocal) {-values {yes no y n 1 0}}
    set optionsRange(-autoReply) {-values {yes no y n 1 0}}
    set optionsRange(-bOk) {-values {yes no y n 1 0}}
    set optionsRange(-bCancel) {-values {yes no y n 1 0}}
    set optionsRange(-bAbort) {-values {yes no y n 1 0}}
    set optionsRange(-modal) {-values {yes no y n 1 0}}
    set optionsRange(-checkAbort) {-values {yes no y n 1 0}}
    ## define allowed options
    set options(-type) "info"
    set options(-forceLocal) no
    set options(-autoReply) yes
    set options(-init) yes
    ## the belows are switched by type
    set options(-imageFile) {}
    set options(-symbolFile) {}
    set options(-title) {}
    set options(-bOk) {}
    set options(-bCancel) {}
    set options(-bAbort) {}
    set options(-bOkText) "OK"
    set options(-bCancelText) "Cancel"
    set options(-bAbortText) "Abort"
    set options(-modal) yes
    set options(-checkAbort) yes
    ## set allowed
    set optionsAllowed [array names options]

    ## read args so we can set defaults for the type
    array set options $args
    ## handle the type
    switch $options(-type) {
        "note" -
        "info" {
            set options(-symbolFile) "info.gif"
            set options(-title) "Operator INFO"
            set options(-bOk) yes
            set options(-bCancel) no
            set options(-bAbort) no
        }
        "question" {
            set options(-symbolFile) "info_question.gif"
            set options(-title) "Operator INFO"
            set options(-bOk) yes
            set options(-bCancel) yes
            set options(-bAbort) no
        }
        "warning" {
            set options(-symbolFile) "warning.gif"
            set options(-title) "Operator WARNING"
            set options(-bOk) yes
            set options(-bCancel) no
            set options(-bAbort) no
        }
        "error" {
            set options(-symbolFile) "error.gif"
            set options(-title) "Operator ERROR"
            set options(-bOk) yes
            set options(-bCancel) no
            set options(-bAbort) no
        }
        default {
            error "operatorOk: Type '$options(-type)' not supported, use note/info, question, warning, error;"
        }
    }
    ## read args to overwrite the defaults set
    array set options $args

    ## check for unallowed/range options
    checkCompAllowed options $optionsAllowed
    checkCompRange options optionsRange

    ## changed interface
    if {$options(-imageFile) != {}} {
        set options(-symbolFile) $options(-imageFile)
    }

    ## check if we are remote - so call EGSE
    if {![valbool $options(-forceLocal)] && [info exists ::SCOE_state]\
        && ($::SCOE_state == "REMOTE") && ([info command scoe_server] != {})} {
        set result [scoe_server operator_ok -text $text -autoReply $options(-autoReply)]
        ## check abort
        if {($result == -1) && [valbool $options(-checkAbort)]} {
            testSequenceAbort
            check_abort
        }
        ## return result
        return $result
    }

    ## check for auto reply
    if {([find object ::COM] != {}) && ([COM getcomp autoReply] != {}) && [valbool $options(-autoReply)]} {
        if {([COM map autoReply [COM getval autoReply]] == "LOCAL") || ([COM map autoReply [COM getval autoReply]] == "ALWAYS")} {
            set result [COM getval opok]
            set mappedResult [COM mapvalues opok $result]
            if {$text != {}} {
                Log n "operatorOk: $text"
            }
            Log n "operatorOk: Local auto reply: COM:OPOK = $mappedResult"
            ## check abort
            if {($result == -1) && [valbool $options(-checkAbort)]} {
                testSequenceAbort
                check_abort
            }
            ## return result
            return $result
        }
    }

    ## create instance of window
    if {![info exists ::SCOE_CONFIG(windows.OpOK)] || ($::SCOE_CONFIG(windows.OpOK) == {})} {
        set ::SCOE_CONFIG(windows.OpOK) [OperatorOkWindow ".#auto"]
        $::SCOE_CONFIG(windows.OpOK) configure -symbolPath [file join $::PICTURES_GPATH opok]
    }
    if {[valbool $options(-init)]} {$::SCOE_CONFIG(windows.OpOK) init}
    $::SCOE_CONFIG(windows.OpOK) configure -symbolFile $options(-symbolFile) -text $text -title $options(-title)
    $::SCOE_CONFIG(windows.OpOK) configure -bOk $options(-bOk) -bCancel $options(-bCancel) -bAbort $options(-bAbort)
    $::SCOE_CONFIG(windows.OpOK) configure -oktext $options(-bOkText) -canceltext $options(-bCancelText) -aborttext $options(-bAbortText)

    if {$options(-title) != {}} {
        $::SCOE_CONFIG(windows.OpOK) configure -title $options(-title)
    }

    ## request operator
    if {[valbool $options(-modal)]} {
        set result [$::SCOE_CONFIG(windows.OpOK) waitForOperator]
    } else {
        $::SCOE_CONFIG(windows.OpOK) configure -show y
        set result 1
    }

    if {($result == -1) && [valbool $options(-checkAbort)]} {
        testSequenceAbort
        check_abort
    }

    ## return result
    return $result
}

proc gen_statefield {w label var states} {
    iwidgets::statefield $w -labeltext $label -labelpos n -width 15 -labelfont $::font
    foreach state $states {
        set stval [lindex $state 0]
        set color [lindex $state 1]
        if {[llength $state] > 2} {
            set text [lindex $state 2]
        } else {
            set text $stval
        }
        $w addstate $stval -text $text -font $::font
        if {$color != {}} {
            $w addstate $stval -text $text -background $color -font $::font
        } else {
            $w addstate $stval -text $text -font $::font
        }
    }
    $w configure -statevar $var
    $w component hull configure -relief raised -borderwidth 1
}

proc gen_labelfield {w text var {labelpos n} {just c} {width 15}} {
    iwidgets::entryfield $w -labeltext $text -labelpos $labelpos -borderwidth 0 \
        -textvariable $var -justify $just -width $width -state disabled -labelfont $::font -relief flat
    $w configure -textbackground [$w cget -background] -textfont $::font
    $w component hull configure -relief sunken -borderwidth 2
}

proc getNObjectWindow {object} {
    if {[$object cget -windowObject] == {}} {
        createNObjectWindow -object $object
    }
    return [$object cget -windowObject]
}

proc showNObjectWindow {object {action {show}}} {
    set window [getNObjectWindow $object]
    showGenericWindow $window $action
}

proc createNObjectWindow {args} {
    Log d "createNObjectWindow: args $args"

    ## -class -object -title
    set options(-title) {}
    set options(-class) {}
    set options(-object) {}
    array set options $args

    if {$options(-title) == {}} {
        set options(-title) [$options(-object) cget -loggingTitle]
    }
    if {$options(-class) == {}} {
        set options(-class) [$options(-object) cget -windowClass]
    }

    ## check for emtpy options - gen error
    foreach option {-class -object} {
        if {$options($option) == {}} {
            error "createNObjectWindow : No $option specified, it's a must"
        }
    }

    ## info
    Log t "$options(-title): Building GUI object, please wait ..."
    update
    ## check time
    set st [getclock -float]

    ## create window object
    set nobwin [$options(-class) ".#auto"]

    ## configure paths and title
    $nobwin configure -title $options(-title) -nobjectObject $options(-object)

    ## build the gui entrys
    $nobwin buildGui

    ## configure the states/modes - they depend on the object after buildGui
    traceButtonState

    ## we are fini
    Log n "$options(-title): Building GUI object finished ([format %.2f [getclock -float -ref $st]] sec.)"
}

proc showGenericWindow {window {action {show}}} {
    Log d "showGenericWindow : window $window"
    set window [string trim $window :]
    switch $action {
        "show" {
            $window configure -show y
            $window toTop
        }
        "hide" {
            $window configure -show n
        }
        "toggle" {
            if {[valbool [$window cget -show]]} {
                $window configure -show n
            } else {
                $window configure -show y
                $window toTop
            }
        }
        default {
            error "showGenericWindow : object '$object' : Unknown action '$action'"
        }
    }
}

proc getFontHeight {window font} {
    return [expr [font metrics $font -displayof $window -linespace] * 1.2]
}


####### POWER-SCOE specific functions ######################################

proc trace_err {widget arg1 arg2 op} {
    global application_background
    if {$arg2!={}} {
    upvar ${arg1}($arg2) err
    } else {
    upvar $arg1 err
    }
    if {[info exists err]} {
    if {[string is integer -strict $err] && $err >0 } {
        if {$err & 1} {
        $widget configure -background red
        } else {
        ## other bits mean warning
        $widget configure -background orange
        }
    } elseif {[valbool $err]} {
        $widget configure -background red
    } else {
        $widget configure -background $application_background
    }
    }

}

proc trace_cote_temp {widget label limitName arg1 arg2 op} {
    global application_background
    if {$arg2!={}} {
    upvar ${arg1}($arg2) temp
    set v ${arg1}_$arg2
    } else {
    upvar $arg1 temp
    set v $arg1
    }
    upvar \#0 $limitName limit
    if {$limit<$temp} {
    Log W "COTE $label is $temp, exceeds soft limit of $limit"
    $widget configure -background orange
    } else {
    $widget configure -background $application_background
    }

}
proc trace_temp_err {widget temp_bit arg1 arg2 op} {
    if {$temp_bit=={}} {set temp_bit 1}
    global application_background
    if {$arg2!={}} {
    upvar ${arg1}($arg2) err
    set v ${arg1}_$arg2
    } else {
    upvar $arg1 err
    set v $arg1
    }
    #Log H "trace_temp_err [list $temp_bit $v $err]"
    if {[info exists err]} {
    if {[string is integer -strict $err] && ($err & (1<<$temp_bit)) >0 } {
        $widget configure -background red
    } else {
        ### check if warning level is exceeded
        if {[info exists ::trace_scoe_temp_warn($v)] && $::trace_scoe_temp_warn($v)} {
        $widget configure -background orange
        } else {
        ##### normal temp
        $widget configure -background $application_background
        }
    }
    }
}

proc trace_scoe_temp {widget label limitName arg1 arg2 op} {
    global application_background
    if {$arg2!={}} {
    upvar ${arg1}($arg2) temp
    set v ${arg1}_$arg2
    } else {
    upvar $arg1 temp
    set v $arg1
    }
    upvar \#0 $limitName limit
    if {$limit<$temp} {
    if {![info exists ::trace_scoe_temp_warn($v)] || !$::trace_scoe_temp_warn($v)} {
        Log W "WARNING - elevated $label $temp degC, exceeds soft limit of $limit degC"
        set ::trace_scoe_temp_warn($v) 1
    } else {
        Log D "WARNING - elevated $label $temp degC, exceeds soft limit of $limit degC"
    }
    if {$widget != {}} {
        if {[$widget cget -background]==$application_background} {
        $widget configure -background orange
        }
    }
    } else {
    set ::trace_scoe_temp_warn($v) 0
    if {$widget != {}} {
        if {[$widget cget -background]=="orange"} {
        ### do not touch if hw OT
        $widget configure -background $application_background
        }
    }
    }

}



proc viwidget {parent label var_v var_i {var_st {}}  \
           {var_err {}} {var_verr {}} {var_ierr {}} {width 5}} {
    set labelmargin 0
    set wname [string tolower $label]
    regsub -all {[( /%:.)]+} $wname _ wname
    set w $parent.vi$wname
    iwidgets::labeledwidget $w -labeltext $label -labelmargin $labelmargin
    set f [$w childsite]
    set l [$w component label]
    $f configure  -borderwidth 0
    iwidgets::entryfield $f.v -textvariable $var_v -labeltext V \
    -labelpos e -relief flat -width $width -justify right -labelmargin 0 \
    -state disabled -borderwidth 0 -disabledforeground black
    $f.v component entry configure  -disabledforeground {}
    pack $f.v -side left  -pady 0 -ipady 0 -padx 0 -ipadx 0
    iwidgets::entryfield $f.a -textvariable $var_i  -labeltext A \
        -labelpos e -relief flat -width $width -justify right -labelmargin 0 \
        -state disabled  -borderwidth 0 -disabledforeground black
    $f.a component entry configure  -disabledforeground {}
    pack $f.a -side left  -pady 0 -ipady 0 -padx 0 -ipadx 0
    if {$var_st!={}} {
    global  $var_st
    trace variable $var_st w "trace_vi [$w component label]\
        $var_st $var_err"
    }
    if {$var_err!={}} {
    global $var_err
    trace variable $var_err w "trace_vi [$w component label]\
        $var_st $var_err"
    }
    if {$var_verr!={}} {
    global $var_verr
    trace variable $var_verr w "trace_err [$f.v component label]"
    }
    if {$var_ierr!={}} {
    global $var_ierr
    trace variable $var_ierr w "trace_err [$f.a component label]"
    }
    return $w
}

proc sas_widget {parent label var_v var_i {var_st {}} {var_sl {}} {width 5}} {
    set labelmargin 0
    set wname [string tolower $label]
    regsub -all {[( /%:.)]+} $wname _ wname
    set w $parent.vi$wname
    iwidgets::labeledwidget $w -labeltext $label -labelmargin $labelmargin
    $w component hull configure -relief ridge -borderwidth 1

    set f [$w childsite]
    set l [$w component label]
    $f configure  -borderwidth 0
    iwidgets::entryfield $f.v -textvariable $var_v -labeltext V \
    -labelpos e -relief flat -width $width -justify right -labelmargin 0 \
    -state disabled -borderwidth 0 -disabledforeground black
    $f.v component entry configure  -disabledforeground {}
    pack $f.v -side left  -pady 0 -ipady 0 -padx 0 -ipadx 0
    iwidgets::entryfield $f.a -textvariable $var_i  -labeltext A \
        -labelpos e -relief flat -width $width -justify right -labelmargin 0 \
        -state disabled  -borderwidth 0 -disabledforeground black
    $f.a component entry configure  -disabledforeground {}
    pack $f.a -side left  -pady 0 -ipady 0 -padx 0 -ipadx 0
    if {$var_st!={}} {
    trace variable $var_st w [list trace_sas_st $w]
    }
    if {$var_sl!={}} {
    trace variable $var_sl w [list trace_sas_sl $w]
    }
    return $w
}

proc trace_sas_st {widget arg1 arg2 op} {
    global application_background
    if {$arg2!={}} {
    upvar ${arg1}($arg2) st
    } else {
    upvar $arg1 st
    }
    # {0 OK 1 MAINT 2 ERROR  3 MERR}
    set f [$widget childsite]
    if {$st & 1} {
    ### maintenance
    $widget configure -foreground white
    $f.v configure -foreground white
    $f.a configure -foreground white
    } else {
    $widget configure -foreground black
    $f.v configure -foreground black
    $f.a configure -foreground black
    }
    if {$st & 2} {
    ### selftest error
    $widget component label configure -background orange
    } else {
    $widget component label configure  -background $application_background
    }
}

proc trace_sas_vars {widget stateVar outputVar v_minVar v_maxVar i_minVar arg1 arg2 op} {
    foreach v {state output v_max v_min i_min} {
    upvar [set ${v}Var] $v
    }

    global application_background
    global sas_limits
    # {0 OK 1 MAINT 2 ERROR  3 MERR}
    set f [$widget childsite]
    if {$state & 1} {
    ### maintenance
    $widget configure -foreground white
    $f.v configure -foreground white
    $f.a configure -foreground white
    } else {
    $widget configure -foreground black
    $f.v configure -foreground black
    $f.a configure -foreground black
    }
    #Log H "$v_min $v_max $i_min"
    if {$state & 2} {
    ### selftest error
    $widget component label configure -background orange
    } else {
    if {($output & 2)} {
        if {$v_min<$sas_limits(v_shunt)+$i_min*$sas_limits(r_harness)} {
        if {abs($v_max)-($v_min)>$sas_limits(switching)} {
            $widget component label configure  -background yellow
        } elseif {$i_min> $sas_limits(on_current)} {
            $widget component label configure  -background cyan
        } else {
            ## not really ON
            $widget component label configure  -background grey
        }
        } elseif  {$i_min> $sas_limits(on_current)} {
        $widget component label configure  -background green
        } else {
        $widget component label configure  -background grey
        }

    } else {
        $widget component label configure  -background $application_background
    }
    }
}

proc trace_sas_sl {widget arg1 arg2 op} {
    global application_background
    #Log H [list trace_sas_sl $widget $arg1 $arg2 $op]
    if {$arg2!={}} {
    upvar  ${arg1}($arg2) st
    } else {
    upvar  $arg1 st
    }
    set f [$widget childsite]

    # bit 0 = ov A141, bit 1 = oc A141, bit 2 = ov Elgar, bit 3 = oc Elgar
    if {$st & 1} {
    [$f.v component entry] configure -disabledbackground red
    } else {
    [$f.v component entry] configure -disabledbackground $application_background
    }
    if {$st & 2} {
    [$f.a component entry] configure -disabledbackground red
    } else {
    [$f.a component entry] configure -disabledbackground $application_background
    }
    if {$st & 4} {
    $f.v configure -background red
    } else {
    $f.v configure -background $application_background
    }
    if {$st & 8} {
    $f.a  configure -background red
    } else {
    $f.a  configure -background $application_background
    }

}



proc v_widget {parent label var_v {var_verr {}} {width 5}} {
    set wname [string tolower $label]
    regsub -all {[ /%:.]+} $wname _ wname
    set w $parent.v$wname
    iwidgets::labeledwidget $w -labeltext $label -labelmargin 8
    set f [$w childsite]
    set l [$w component label]
    $f configure  -borderwidth 0
    iwidgets::entryfield $f.v -textvariable $var_v -labeltext V \
    -labelpos e -relief flat -width $width -justify right -labelmargin 0 \
    -state disabled -borderwidth 0  -disabledforeground black
    $f.v component entry configure  -disabledforeground {}
    pack $f.v -side left  -pady 0 -ipady 0 -fill x -expand 1
    if {$var_verr!={}} {
    global $var_verr
    trace variable $var_verr w "trace_err [$f.v component label]"
    }
    return $w
}

proc i_widget {parent label var_i {var_ierr {}} {width 5}} {
    set wname [string tolower $label]
    regsub -all {[ /%:.]+} $wname _ wname
    set w $parent.i$wname
    iwidgets::labeledwidget $w -labeltext $label -labelmargin 8
    set f [$w childsite]
    set l [$w component label]
    $f configure  -borderwidth 0
    iwidgets::entryfield $f.a -textvariable $var_i -labeltext A \
    -labelpos e -relief flat -width $width -justify right -labelmargin 0 \
    -state disabled -borderwidth 0  -disabledforeground black
    $f.a component entry configure  -disabledforeground {}
    pack $f.a -side left  -pady 0 -ipady 0 -fill x -expand 1
    if {$var_ierr!={}} {
    global $var_ierr
    trace variable $var_ierr w "trace_err [$f.a component label]"
    }
    return $w
}

proc other_widget {parent label var_o unit {var_oerr {}} {width 5}} {
    set wname [string tolower $label]
    regsub -all {[ /%:.]+} $wname _ wname
    set w $parent.v$wname
    iwidgets::labeledwidget $w -labeltext $label -labelmargin 8
    set f [$w childsite]
    set l [$w component label]
    $f configure  -borderwidth 0
    iwidgets::entryfield $f.v -textvariable $var_o -labeltext $unit \
        -labelpos e -relief flat -width $width -justify right -labelmargin 0 \
        -state disabled -borderwidth 0  -disabledforeground black
    $f.v component entry configure  -disabledforeground {}
    pack $f.v -side left  -pady 0 -ipady 0 -fill x -expand 1
    if {$var_oerr!={}} {
        global $var_oerr
        trace variable $var_oerr w "trace_err [$f.v component label]"
    }
    return $w
}

image create bitmap degCbitmap  -data {
    #define degc.bmp_width 14
    #define degc.bmp_height 14
    static unsigned char degc.bmp_bits[] = {
    0x00, 0x00, 0x00, 0x00, 0x06, 0x00, 0x89, 0x17, 0xc9, 0x18, 0x66, 0x10,
    0x60, 0x10, 0x60, 0x00, 0x60, 0x00, 0x60, 0x00, 0x60, 0x10, 0xc0, 0x18,
    0x80, 0x0f, 0x00, 0x00};

}

image create bitmap degCbitmap_fixed  -data {
    #define degC_width 16
    #define degC_height 16
    static char degC_bits[] = {
    0x00, 0x00, 0x00, 0x00, 0x1C, 0x00, 0x14, 0x0F, 0x9C, 0x10, 0x40, 0x00,
    0x40, 0x00, 0x40, 0x00, 0x40, 0x00, 0x40, 0x00, 0x80, 0x10, 0x00, 0x0F,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, };
}

proc t_widget {parent label var_t {var_terr {}} {temp_bit 1} {degCbitmap {}}} {
    set wname [string tolower $label]
    regsub -all {[ /%:.]+} $wname _ wname
    set w $parent.t$wname
    iwidgets::labeledwidget $w -labeltext $label -labelmargin 8
    set f [$w childsite]
    set l [$w component label]
    $f configure  -borderwidth 0
    ## -labeltext {Deg.C}
    ##  -labelimage degCbitmap
    iwidgets::entryfield $f.t -textvariable $var_t \
    -labelpos e -relief flat -width 5 -justify right -labelmargin 0 \
    -state disabled -borderwidth 0  -disabledforeground black
    if {$degCbitmap=={}} {
    $f.t configure -labeltext {deg.C}
    } else {
    $f.t configure  -labelimage $degCbitmap
    }
    $f.t component entry configure  -disabledforeground {}
    pack $f.t -side left  -pady 0 -ipady 0  -fill x -expand 1
    if {$var_terr!={}} {
    global $var_terr
    #trace variable $var_terr w "trace_err [$f.t component label]"
    trace variable $var_terr w "trace_temp_err $w $temp_bit"
    }
    return $w
}


####################################################################################################
## The following procs (or their API) MUST stay in at global level for backward compatibility

## Log will be specific for each project
#proc Log {level text args} {
#    if {[info exists ::SCOE_CONFIG(scoe.LogIfObj)] && $::SCOE_CONFIG(scoe.LogIfObj) != {}} {
#        eval $::SCOE_CONFIG(scoe.LogIfObj) add [list $level] [list $text] $args
#    } else {
#        puts $text
#    }
#}

proc operator_ok {text {show_abort 1}} {
    return [operatorOk $text -bAbort $show_abort -type question]
}

proc local_operator_ok  {msg {show_abort 1}} {
    if {![winfo exists .ok_message]} {
    if {![info exists ::font]} {
        set ::font fixed
    }
    iwidgets::messagedialog .ok_message -modality application \
        -title "Operator Confirmation" \
        -bitmap questhead -padx 20 -pady 20
    .ok_message add Abort -text Abort -font $::font \
        -command "set ::operator_ok_var -1;.ok_message deactivate -1"
    .ok_message buttonconfigure Cancel -text Cancel -font $::font \
        -command "set ::operator_ok_var 0;[.ok_message buttoncget Cancel -command]"
    .ok_message buttonconfigure default -font $::font \
        -command "set ::operator_ok_var 1;[.ok_message buttoncget default -command]"

    bind  .ok_message <F8> {
        Log w "Operator request - Disabled modality"
        grab release .ok_message
        .ok_message configure -modality none
    }
    bind  .ok_message <space> {
        .ok_message invoke
    }
    }
    .ok_message configure -text $msg -font $::font
    if {$show_abort} {
    .ok_message  buttonconfigure  Abort -state normal
    } else {
    .ok_message  buttonconfigure  Abort -state disabled
    }
    .ok_message center


    set ::operator_ok_var -1
    set result [.ok_message activate]

    if {$result == {}} {
    ## in case modality is none
    tkwait variable ::operator_ok_var
    set result $::operator_ok_var
    }

    if {$result>0} {
    Log N "Operator Acknowledge for : $msg"
    } else {
    if {$result<0} {
        error "Operator Abort for : $msg"
    } else {
        Log N "Operator Negative Acknowledge for : $msg"
    }
    }
    return $result
}

proc local_operator_comment { prompt {abort_val {}} } {
    global op_comment font
    if {![info exists ::font]} {
    set ::font fixed
    }

    Log D "local_operator_comment $prompt"
    if { ![winfo exists .comment_message] } {
    iwidgets::dialog .comment_message -modality application \
        -title "Operator Comment" \
        -padx 20 -pady 20
    .comment_message buttonconfigure Cancel -text Abort -font $font
    .comment_message hide Apply
    .comment_message hide Help
    set parent [.comment_message childsite].f
    frame $parent
    pack $parent
    label $parent.l -font $font
    pack $parent.l
    entry $parent.comment -textvariable op_comment -font $font
    pack $parent.comment -fill x -expand yes
    }
    set op_comment ""
    set parent [.comment_message childsite].f
    $parent.l configure -text $prompt
    .comment_message center
    if {[.comment_message activate]} {
    return $op_comment
    } else {
    return $abort_val
    }
}

proc test_operator_ok {msg {log_lvl t}} {
    if {[info exists ::operator_test_halt] && $::operator_test_halt} {
        Log $log_lvl "### test_operator_ok : $msg"
        append msg "\n\nPress OK to continue testing with pausing, press Cancel to to disable future halts.\nAbort will stop the command."
        set ::operator_test_halt [expr {[operatorOk $msg -type question -bAbort yes] > 0}]
    } else {
        # assume OK
        Log $log_lvl "### test_operator_ok : Disabled halts : $msg"
        set ::operator_test_halt 0
    }
    return $::operator_test_halt
}

### the function of teh next procedure needs to be checked (if it is still necessary for X-t plotting)
proc MMIset {name value} {
    if {$name!={}} {
    uplevel #0 set "{@scope :: $name}" [list $value]
    ### LMNotify $name  -- equivalent code see below:
    uplevel #0 set "{@scope :: LMchanged($name)}" 1
        ### mark time of last update
    global xt_start_time
    uplevel #0 set "{@scope :: dau_upd_time($name)}" [getclock -float -ref $xt_start_time]
    }
}

proc showToplevel {name {visible {}}} {
    global allowed_windows

    # check visibility
    if {$visible != {}} {
        set ::window_visible($name) [valbool $visible]
    }
    if {[array names ::window_visible $name] == {}} {
        set ::window_visible($name) 1
    }

    # check for correct object
    if {[find objects $name -isa NObject] != {}} {
        set w [getNobjectWindow $name]
        $w configure -show $::window_visible($name)
    } elseif {[find objects $name -isa GenericWindow] != {}} {
        $name configure -show $::window_visible($name)
    } else {

        set w $name

        if {![winfo exists $w]} {
            Log N "Generating $name window ... please wait"
            ShowWindow$w
            Log T "$name window done"

            if {[wm protocol $w WM_DELETE_WINDOW] == {}} {
                # only do this if no action has been set
                wm protocol $w WM_DELETE_WINDOW "showToplevel $name 0"
            }
            if {[info exists ::window_geometry($w)] && $::window_geometry($w)!={} } {
                wm geometry $w $::window_geometry($w)
            }
        } elseif {$w == ".top${::SCOE}_Synoptic"} {
        if {$visible == {}} {
        set ::window_visible($w) 1
        } else {
        set ::window_visible($w) 0
        }
        #ShowWindow$w
        if {$::window_visible($w)} {
        set ::active.rfSynoptic 1
        activatecanvas ::active.rfSynoptic
        } else {
        set ::active.rfSynoptic 0
        activatecanvas ::active.rfSynoptic
        }
    }

        set ret 0 ;# needed for dialogs
        if {$::window_visible($w)} {
            if {[object_isa $w Shell]} {
                if {[catch "$w activate" msg]} {
                    Log W $msg
                } else {
                    set ret $msg
                }
            } else {
                wm deiconify $w
            }
        } else {
            if {[object_isa $w Shell]} {
                $w deactivate 0
            } else {
                wm withdraw $w
            }
        }
        return $ret
    }
}