## source $::UTIL_GPATH/util.tcl
# ------------------------------------------------------------------------------
# global used
#
# FEEDBACK_WIDGET
# SCOE_CONFIG(scoe.LogIfObj)
# LOGGING_WRAPPER_DIS_CLASS
# LOGGING_WRAPPER_SHIFT_OBJ
# {ns}_NAMESPACEVARS
# ------------------------------------------------------------------------------

proc documentNObjects {args} {
    set options(-filename) {}
    set options(-objects) {}
    array set options $args

    if {$options(-filename) != {}} {
        set f [open $options(-filename) w]
    }

    foreach obj $options(-objects) {
    set loggingTitle $obj
    catch {set loggingTitle [$obj cget -loggingTitle]}
        if {$options(-filename) == {}} {
            set objClass [namespace tail [$obj info class]]
        set filename [file join $::CONFIG_GPATH ${loggingTitle}_${objClass}.txt]
            set f [open $filename w]
        } else {
            set filename $options(-filename)
        }
        eval $obj csvWrite -file $f $args
        if {$options(-filename) == {}} {
            close $f
        } else {
            puts $f ""
            puts $f ""
            puts $f ""
        }
        Log n "$loggingTitle: Written scsv to $filename"
    }

    if {$options(-filename) != {}} {
        close $f
    }
}

proc convertBase {values base newbase} {
    if {($base == $newbase) || ($values == {})} {return $values}

    set vallist {}
    foreach value $values {
        if {[string is double $value] && ($value != {})} {
            lappend vallist [expr {$value * ([getBase $base] / [getBase $newbase])}]
        } else {
            lappend vallist $value
        }
    }
    return $vallist
}

proc convertRange {definition newbase} {
    Log d "convertRange: definition $definition newbase $newbase"
    set range(-minmax) {}       ;# min to max range : -minmax {30:40 50.4:50.9}
    set range(-values) {}       ;# list of values that are OK : -values {AUTO BASE 10 20}
    set range(-regexp) {}       ;# value will be evaluated by regexp : -regexp *[abc]*
    set range(-case)   {}       ;# case sensitivity
    set range(-base)   {}       ;# base of values, minmax
    array set range $definition

    set rangeMinMax {}
    foreach minmax $range(-minmax) {
        set minmax [split $minmax :]
        set min [lindex $minmax 0]
        set max [lindex $minmax 1]

        set min [convertBase $min $range(-base) $newbase]
        set max [convertBase $max $range(-base) $newbase]

        lappend rangeMinMax "$min:$max"
    }

    set rangeValues [convertBase $range(-values) $range(-base) $newbase]

    # now put it all together now
    set retStr {}
    if {$rangeMinMax != {}} {lappend retStr "-minmax" $rangeMinMax}
    if {$rangeValues != {}} {lappend retStr "-values" $rangeValues}
    if {$range(-regexp) != {}} {lappend retStr "-regexp" $range(-regexp)}
    if {$range(-case) != {}} {lappend retStr "-case" $range(-case)}
    if {($retStr != {}) && ($newbase != {})} {lappend retStr "-base" $newbase}

    return $retStr
}

proc createScoped {object name} {
    #Log d "createScoped : object $object name $name"
    set varName [$object info class]::$name
    catch {set varName [$object info variable $name -name]}
    return [list @itcl $object $varName]
}

proc splitScoped {variable} {
    if {[string range $variable 0 4] == "@itcl"} {
        # @itcl ::.tM_nobject0 ::TM_nobject::downlink_cf
        set splitvar [split [lindex $variable 2] {: }]
        set itcl [lindex $variable 0]
        set object [lindex $variable 1]
        set class [join [lrange $splitvar 0 end-2] :]
        set name [lindex $splitvar end]
        return [list -object $object -class $class -name $name]
    } else {
        set name [lindex [split $variable :] end]
        return [list -object {} -class {} -name $name]
    }
}

proc getOptionNumeric {list args} {
    set newArgs {}
    foreach arg $args {
    catch {set arg [expr $arg]}
    lappend newArgs $arg
    }
    set newList {}
    foreach {param value} $list {
    catch {set param [expr $param]}
    lappend newList $param $value
    }
    return [eval getOption [list $newList] $newArgs]
}

proc listMirror {list} {
    set mirror {}
    for {set index [expr [llength $list] - 1]} {$index >= 0} {incr index -1} {
        lappend mirror [lindex $list $index]
    }
    return $mirror
}

proc getOption {list args} {
    foreach arg $args {
        array set optlist $list
        set list {}
        catch {set list $optlist($arg)}
    }
    return $list
}

proc getBases {} {
    return {n u m {} k M G}
}

proc getBaseUnitFormat {definition} {
    set options(-autoformat) yes
    set options(-base) {}
    set options(-unit) {}
    set options(-resolution) {}
    set options(-format) {}
    array set options $definition

    if {$options(-format) != {}} {return $options(-format)}
    if {![valbool $options(-autoformat)] || ($options(-unit) == {})} {return {}}

    set count [expr int(log10([getBase $options(-base)]))]

    set min 0
    switch [string tolower $options(-unit)] {
        "m" {set min 2}
        "%" {set min 2}
        "db" -
        "dbc" -
        "dbm" -
        "dbw" -
        "w" {set min 3}
        "s" -
        "sec" -
        "sec." -
        "seconds" {set min 3}
        "points" {set count 0}
        "rad" {set min 3}
        "degree" {set min 3}
        "rms" {set min 3}
        "hz" {set min 0}
        "hz/sec" {set min 0}
        default {set min 0}
    }

    if {$options(-resolution) != {}} {set min $options(-resolution)}
    incr count $min

    if {$count < 0} {set count 0}
    return "%.${count}f"
}

proc getBase {base} {
    switch $base {
        n   {return            0.000000001}
        u   {return            0.000001}
        m   {return            0.001}
        {}  {return          1.0}
        k   {return       1000.0}
        M   {return    1000000.0}
        G   {return 1000000000.0}
        default {error "getBase : Base '$base' not valid"}
    }
}

proc checkRange {values range {base {}}} {
    ## method returns 1 if check succeeds
    #Log d "checkRange: values $values options $options base $base"

    ## if one of the element is true the method succeeds
    set options(-minmax) {}       ;# min to max range : -minmax {30:40 50.4:50.9}
    set options(-values) {}       ;# list of values that are OK : -values {AUTO BASE 10 20}
    set options(-regexp) {}       ;# value will be evaluated by regexp : -regexp *[abc]*
    set options(-command) {}      ;# command to evaluate

    ## manipulation flags
    set options(-case) yes        ;# case sensitivity
    set options(-list) no         ;# allow values as a list
    set options(-base) {}         ;# base of values, minmax
    set options(-invert) no       ;# invert result

    set options(-isint) no        ;# check if value is of type integer
    array set options $range

    ## specific command for range checking
    if {$options(-command) != {}} {
        return [eval [list $options(-command) $values $range $base]]
    }

    ## if every range check option is empty
    if {($options(-values) == {}) && ($options(-minmax) == {}) && ($options(-regexp) == {})} {
        if {[valbool $options(-invert)]} {return 0}
        return 1
    }

    ## check if value is a list, if not - wrap it
    if {![valbool $options(-list)] || ([llength $values] == 0)} {
        set values [list $values]
    }

    ## each value must be ok
    set valuesCheckedOk 0
    foreach value $values {

        ## check if its numeric
        set isnumeric [string is double $value]
        if {$value == {}} {set isnumeric 0}

        ## convert base
        set value [convertBase $value $base $options(-base)]

        ## check if -case
        set options(-case) [valbool $options(-case)]
        if {!$options(-case)} {set value [string toupper $value]}

        ## check min:max
        if {$isnumeric} {
        
            ## check integer
            if {[valbool $options(-isint)]} {
                if {![string is integer $value]} {continue}
            }
            
            foreach element $options(-minmax) {
                set minmax [split $element :]
                set min [lindex $minmax 0]
                set max [lindex $minmax 1]
                if {($min == {-}) && ($value <= $max)} {
                    incr valuesCheckedOk
                    continue
                }
                if {($value >= $min) && ($max == {+})} {
                    incr valuesCheckedOk
                    continue
                }
                if {($value >= $min) && ($value <= $max)} {
                    incr valuesCheckedOk
                    continue
                }
            }
        }

        ## check values
        foreach element $options(-values) {
            if {!$options(-case)} {set element [string toupper $element]}
            if {$value == $element} {
                incr valuesCheckedOk
                continue
            }
        }

        ## check regexp
        foreach element $options(-regexp) {
            if {$options(-case)} {
                set ret [regexp $element $value]
            } else {
                 set ret [regexp -nocase $element $value]
            }
            if {$ret} {
                incr valuesCheckedOk
                continue
            }
        }
    }

    ## invert?
    set result [expr $valuesCheckedOk >= [llength $values]]
    set result [expr [valbool $options(-invert)] ? !$result : $result]

    ## fini
    return $result
}

proc getCursorPower {value cursor} {
    ## convert to lower case for exponent and convert to real
    set value [expr {[string tolower $value] + 0.0}]

    set exponent 0
    if {[string first e $value] >= 0} {
        set exponent [lindex [split $value e] 1]
        set value [lindex [split $value e] 0]
    }

    ## thats the cursor power
    set power [expr {pow(10,[string first . $value] - $cursor)}]
    ## add exponent
    set power [expr {$power * pow(10,$exponent)}]

    # fini
    return $power
}

# format time for file or for logging
proc formatTime {args} {
    set sorted [extractCompArgs {comp-comp opt-opt} $args]
    set seconds [lindex [getOption $sorted -components] 0]
    set options [getOption $sorted -options]
    if {$seconds == {}} {set seconds [getclock -float]}
    if {[llength $seconds] == 2} {set seconds [expr [lindex $seconds 0] + [lindex $seconds 1]]}

    set forFile 0
    if {[lsearch $options -forfile] != -1} {set forFile 1}
    set forCount 0
    if {[lsearch $options -forcount] != -1} {set forCount 1}
    set withDate 0
    if {[lsearch $options -withdate] != -1} {set withDate 1}
    set withTime 0
    if {[lsearch $options -withtime] != -1} {set withTime 1}
    set withMs 0
    if {[lsearch $options -withms] != -1} {set withMs 1}
    set withGmt 0
    if {[lsearch $options -withgmt] != -1} {set withGmt 1}

    if {$withDate && $withTime} {
        if {$forFile == 1} {
            set format "%Y-%m-%d__%H_%M_%S"
        } else {
            set format "%Y-%m-%d %H:%M:%S"
        }
    } elseif {$withTime} {
        if {$forFile} {
            set format "%H_%M_%S"
        } else {
            set format "%H:%M:%S"
        }
    } elseif {$withDate} {
        if {$forFile} {
            #set format "%d_%m_%Y"
            set format "%Y_%m_%d"
        } else {
            #set format "%d-%m-%Y"
            set format "%Y-%m-%d"
        }
        # if its only for data - skipp ms
        set withMs 0
    } else {
        return {}
    }

    # split mseconds and seconds
    set time [expr int($seconds)]
    set ms [expr int(($seconds - $time) * 100)]
    set ms [format %02i $ms]

    # format time for first time
    if {!$forCount} {
        set ftime [clock format $time -format $format -gmt $withGmt]
    } else {
        # hour can have more than 24 - so we have to calc it by our own
        set hours [expr int($time / 3600.)]
        set minutes [expr int($time / 60. - $hours * 60)]
        set seconds [expr $time - $hours * 3600 - $minutes * 60]
        if {$forFile == 1} {
            set ftime [format %02i_%02i_%02i $hours $minutes $seconds]
        } else {
           set ftime [format %02i:%02i:%02i $hours $minutes $seconds]
       }
    }

    # if we want to have mseconds
    if {$withMs} {
        if {$forFile} {
            set ftime "${ftime}_$ms"
        } else {
            set ftime "${ftime}.$ms"
        }
    }

    return $ftime
}

proc execAsync {args} {
    ## set defaults
    set options(-command) {}
    set options(-timeout) 1000

    ## read args
    array set options $args

    ## exec command in pipe
    set pipeId [open |$options(-command) r+]
    fconfigure $pipeId -blocking 0 -buffering full

    ## create fileevent and timeout
    set ::$pipeId -1
    fileevent $pipeId readable [list set ::$pipeId 1]
    set timeoutId [after $options(-timeout) [list set ::$pipeId 0]]

    ## wait till set
    set data {}
    while {![eof $pipeId] && ([set ::$pipeId] != 0)} {
        vwait ::$pipeId
        append data [string tolower [read $pipeId]]
    }

    ## close the open pipe
    fileevent $pipeId readable {}
    catch {close $pipeId}
    catch {after cancel $timeoutId}

    ## check - fini
    if {[set ::$pipeId] == 0} {
        return {}
    } else {
        return $data
    }
}

proc beep {args} {
    set options(-freq) 2000
    set options(-time) 1000
    set options(-amplitude) 1e6
    set options(-rate) 22000
    array set options $args

    set len [expr round($options(-rate) / (1. / ($options(-time) / 1000.)))]
    set snackFilter [snack::filter generator $options(-freq) $options(-amplitude) 0.0 sine $len]
    set snackSound [snack::sound -rate $options(-rate)]
    $snackSound play -filter $snackFilter -command "$snackSound destroy; $snackFilter destroy"
}

proc valbool {text} {
    set text [string toupper [string trim $text]]
    switch -exact -- $text {
        YES - Y - ON - OK - TRUE - START - EDIT - ENABLE - 1 {return 1}
    }
    return 0
}

proc boolstring {val} {
    if {[valbool $val]} {return "ON"}
    return "OFF"
}

proc fb {args} {
    if {[info exists ::FEEDBACK_WIDGET]} {
        eval $::FEEDBACK_WIDGET $args
    }
}

proc listMax {list} {
    set max [lindex $list 0]
    foreach item $list {
        if {$item > $max} {set max $item}
    }
    return $max
}

proc listMin {list} {
    set min [lindex $list 0]
    foreach item $list {
        if {$item < $min} {set min $item}
    }
    return $min
}

proc listExclude {list exclude} {
    set newList {}
    foreach element $list {
        if {[lsearch $exclude $element] == -1} {
            lappend newList $element
        }
    }
    return $newList
}

proc listMirror {list} {
    set mirror {}
    for {set index [expr [llength $list] - 1]} {$index >= 0} {incr index -1} {
        lappend mirror [lindex $list $index]
    }
    return $mirror
}

proc listRepeat {list count} {
    set newList {}
    while {$count > 0} {
        lappend newList $list
        incr count -1
    }
    return $newList
}

proc listIndices {list} {
    set newList {}
    foreach {index value} $list {
        lappend newList $index
    }
    return $newList
}

proc textMirror {text} {
    set mtext ""
    for {set index [expr [string length $text] - 1]} {$index >= 0} {incr index -1} {
        append mtext [string range $text $index $index]
    }
    return $mtext
}

proc appendMinusOptionVal {optionVals} {
    set result {}
    foreach {option value} $optionVals {
        lappend result -$option $value
    }
    return $result
}

proc removeMinusOptionVal {optionVals} {
    set result {}
    foreach {option value} $optionVals {
        lappend result [string trimleft $option -] $value
    }
    return $result
}

proc removeMinusOption {options} {
    set result {}
    foreach option $options {
        lappend result [string trimleft $option -]
    }
    return $result
}

# creates dummy server ports and leaves them open
proc dummyServerPorts {args} {
    foreach port $args {
        if {$port == {}} {continue}
        if {[catch {set sock [socket -server dummy $port]}]} {
            return 0
        }
    }
    return 1
}

proc arrayMirror {aname} {
    upvar $aname var
    set values {}
    foreach index [array names var] {
        lappend values $var($index) $index
    }
    return $values
}

proc arrayValues {aname} {
    upvar $aname var
    set values {}
    foreach index [array names var] {
        lappend values $var($index)
    }
    return $values
}

proc arrayGetIndex {aname value} {
    upvar $aname var
    foreach index [array names var] {
        if {$value == $var($index)} {return $index}
    }
    error "No such value in array"
}

proc removetags {window tags} {
    bindtags $window [listExclude [bindtags $window] $tags]
}

proc appendtags {window tags} {
    bindtags $window [concat [bindtags $window] $tags]
}

proc getDidYouMeanString {value comparators args} {
    if {$value == {}} {error "getDidYouMeanString : Missing value parameter"}
    if {$comparators == {}} {error "getDidYouMeanString : Missing comparators parameter"}
    # read out options
    set possibles [eval getComparisonProbabilitys [list $value $comparators] $args]
    #puts $possibles
    set possible [lindex $possibles 0]
    set didYouMean [string trim [lindex $possible 0] :]
    set probability [format %0.f [lindex $possible 1]]
    set valueTypo [string trim [lindex $possible 2] :]
    return "Did you mean '$didYouMean' (it fits to $probability %), check spelling of '$valueTypo'"
}
proc getComparisonProbabilitys {value comparators args} {
    set compResults {}
    foreach comparator $comparators {
        set compResult [eval getComparisonProbabilityDouble [list $value $comparator] $args]
        set valueProbability [lindex $compResult 0]
        set valueErrorString [lindex $compResult 1]
        lappend compResults [list $comparator $valueProbability $valueErrorString]
    }
    set compResults [lsort -real -index 1 -decreasing $compResults]
    return $compResults
}
proc getComparisonProbabilityDouble {value comparator args} {
    set compResult1 [eval getComparisonProbability [list $value $comparator] $args]
    set valueProbability1 [lindex $compResult1 0]
    set valueErrorString1 [lindex $compResult1 1]
    set compResult2 [eval getComparisonProbability [list $comparator $value] $args]
    set valueProbability2 [lindex $compResult2 0]
    set valueErrorString2 [lindex $compResult2 1]
    return [list [expr ($valueProbability1 + $valueProbability2) / 2.] $valueErrorString1]
}
# return probability that value is equal to comparator
# value=downlink_Freqency comparator=downlink_Frequency -group=1
# returns {94.1176470588 downlink_Freq?ency}
proc getComparisonProbability {value comparator args} {
    set options(-group) 1
    # read out options
    array set options $args

    # safety check
    if {![checkRange $options(-group) {-minmax 1:3}]} {
        error "timetech_rx$this:lock : Wrong group '$options(-group)', must be 0:10"
    }

    # group the values
    set valueGroups {}
    set valueDummy $value
    set position 0
    while {$valueDummy != ""} {
        lappend valueGroups [string range $valueDummy 0 [expr $options(-group) - 1]] $position
        set valueDummy [string range $valueDummy $options(-group) end]
        incr position $options(-group)
    }

    # search each group in value
    set cumulativeShift 0
    set summError 0
    set valueErrorString ""
    set openError 0
    foreach {valueGroup valuePos} $valueGroups {
        set cumulativeValuePos [expr $valuePos + $cumulativeShift]
        set searchAt [expr int(max(0,$cumulativeValuePos - $options(-group)))]
        set foundAt [string first $valueGroup $comparator $searchAt]
        set valueShift [expr $foundAt - $cumulativeValuePos]
        set absError [expr int(min($options(-group),abs($valueShift)))]
        if {$absError} {
            if {!$openError} {append valueErrorString "?"}
            set openError 1
            append valueErrorString $valueGroup
        } else {
            set openError 0
            append valueErrorString $valueGroup
        }
        if {$foundAt < 0} {
            set valueShift 0
        } elseif {$valueShift < 0} {
            set valueShift [expr $absError * -1]
        } else {
            set valueShift $absError
        }
        incr cumulativeShift $valueShift
        incr summError $absError
        #puts "Group: $valueGroup ValuePos: $cumulativeValuePos SearchAt: $searchAt FoundAt: $foundAt ValueShift: $valueShift AbsError: $absError"
    }
    set maxError [expr [llength $valueGroups] / 2. * $options(-group)]
    set valueProbability [expr ($maxError - $summError) / $maxError * 100.]
    return [list $valueProbability $valueErrorString]
}

proc wait_ms {waitTime {doStep 0}} {
    set stepTime 500
    set nowTime [getclock -float]
    set endTime [expr $nowTime + $waitTime / 1000.0]
    set approxLoops [expr int(($endTime - $nowTime) / ($stepTime / 1000.))]
    if {$doStep} {set fbid [fb new $approxLoops]}
    while {[set remainingTime [expr {$endTime - [getclock -float]}]] > 0.5} {
        check_abort
        after $stepTime
        if {$doStep} {fb step $fbid}
    }
    if {$remainingTime > 0} {
        after [expr int($remainingTime * 1000)]
    }
    if {$doStep} {fb exit $fbid}
}

proc expandIndex {index} {
    #| converts an index expression as e.g  1,4,7:10,20:30:5                  |
    #| into the index list  {1 4 7 8 9 10 20 25 30}                           |
    #| If a range a:b:c is specified, a, b and c must be integers             |
    if {![regexp {[,:]} $index]} {
        ### single index
        return $index
    }
    set results {}
    foreach item [split $index ,] {
        if {[llength [set range [split $item :]]] < 2} {
            ### no range specified
            lappend results $item
        } elseif {[llength $range] > 3} {
            error "expandIndex : Invalid index range '$item' in '$index'"
        } else {
            ### we have 2 or 3 elements separated by colons
            if {[llength $range] == 2} {
                foreach {start stop} $range {}
                set incr 1
            } else {
                foreach {start stop incr} $range {}
            }
            if {![string is integer $start] || ![string is integer $stop] || ![string is integer $incr]} {
                error "expandIndex : Non integer index range '$item' in '$index'"
            }
            for {set i $start} {$i <= $stop} {incr i $incr} {
                lappend results $i
            }
        }
    }
    return $results
}

proc unexpandIndex {index} {
    #| converts an index index list as e.g   {1 4 7 8 9 10 20 21 22 23 25 30}
    #| into the expression 1,4,7:10,20:23,25,30

    set results ""
    set last {}
    set in_sequence 0
    set itemList {}
    foreach item $index {
	set itemList [concat $itemList [expandIndex $item]]
    }

    foreach item [lsort -integer $itemList] {
    if {$last=={}} {
        set results $item
    } elseif {$item==$last+1} {
        ## in sequence
        set in_sequence 1
    } else {
        ## break in sequence
        if {$in_sequence} {
        append results ":$last,$item"
        } else {
        append results ",$item"
        }
        set in_sequence 0
    }
    set last $item
    }
    if {$in_sequence} {
    append results ":$last"
    }
    return $results
}



proc expandData {index data} {
    #| expands data according to an index expression                          |
    #| if the data is a single item, it is repeated as requitred              |
    #| if the data is a list, its length must agree with the index expression |
    set index_list [expandIndex $index]
    set result {}
    if {[llength $data] == 1} {
        ### one value for all
        foreach ix $index_list {
            lappend result $data
        }
    } elseif {[llength $data] == [llength $index_list]} {
        ### individual items provided
        set result $data
    } else {
        error "expandData : Data to index mismatch, needed [llength $index_list] items for data, got only [llength $data] data items"
    }

    return $result
}
proc selectData {index data} {
    #| select data from list  according to an index expression                          |
    #| the length of  data must be at least the maximum expanded index value
    set index_list [expandIndex $index]
    set result {}
    set nd [llength $data]
    foreach ix $index_list {
    if {$ix > $nd} {
        error "selectData : Data to index mismatch, needed $ix items for data, got only $nd data items"
    }
    lappend result [lindex $data [expr {$ix-1}] ]
    }
    return $result
}

proc getColorRGB {window color} {
    set rgb [winfo rgb $window $color]
    return [list [expr [lindex $rgb 0]/256] [expr [lindex $rgb 1]/256] [expr [lindex $rgb 2]/256]]
}

proc LogEv {level txt args} {
    global in_fileevent
    set in_fileevent 1
    set cmd [lreplace $args -1 -1  Log $level " $txt"] ;# put a space in front to identify  LogEv in logfile
    eval $cmd
    #Log $level $txt $args
    set in_fileevent 0
}

proc unparse {parameter_list} {
    set result ""
    foreach {param paramValue} $parameter_list {
        if {$result != ""} {append result " "}
        set values ""
        foreach value $paramValue {
            if {$values != ""} {append values ","}
            if {([string first " " $value] >= 0) || ([string first "," $value] >= 0)} {
                append values "\"$value\""
            } elseif {$value != {}} {
                append values $value
            } else {
                append values "\"\""
            }
        }
        if {$values != {}} {
            append result "$param=$values"
        } else {
            append result "$param=\"\""
        }
    }

    ## fini
    return $result
}

proc checkCCSet {parameter_string range {base {}}} {
    ## build a list out of the string
    set parameter_list [parse $parameter_string]

    ## if we are running with CCS_sim return true
    if {[catch {set ::EGSE_simulator} msg] || ($msg == 1)} {return 1}

    ## check objects in checkCCReport
    checkCCReport [listIndices $parameter_list] $range $base

    ## check values according to object
    foreach {param paramValue} $parameter_list {
        ## extract object:param
        set index [string first : $param]
        set paramObject [string range $param 0 [expr $index - 1]]
        set paramName [string range $param [expr $index + 1] end]
        
        ## check for array access
        set iname {}
        set array_access [regexp {(.*)\((.*)\)$} $paramName dummy aname iname]
        if {$array_access} {
            set paramName $aname
        }

        ## get global basic object
        set basicObject [find objects ::$paramObject -isa basic]

        ## check if value is in range
        if {![$basicObject checkval $paramName $paramValue]} {
            set paramRange [convertRange [$basicObject getcomp $paramName -range] [$basicObject getcomp $paramName -base]]
            set paramValueMap [$basicObject getcomp $paramName -valuemap]
            set msg "Parameter '$param': Value '$paramValue' out of range: '$param' range $paramRange"
            if {$paramValueMap != {}} {
                append msg ": Valuemap: $paramValueMap"
            }
            error $msg
        }
    }

    ## no error
    return 1
}

proc checkCCReport {parameter_string range {base {}}} {
    ## build a list out of the string
    set parameter_list [parse $parameter_string]

    ## if we are running with CCS_sim return true
    if {[catch {set ::EGSE_simulator} msg] || ($msg == 1)} {return 1}

    ## check each object
    foreach {param paramValue} $parameter_list {
        ## extract object:param
        set index [string first : $param]
        if {$index < 0} {
            error "Missing object:parameter separation ':'. Parameter must be defined as object:component."
        }
        set paramObject [string range $param 0 [expr $index - 1]]
        set paramName [string range $param [expr $index + 1] end]
        
        ## check for array access
        set iname {}
        set array_access [regexp {(.*)\((.*)\)$} $paramName dummy aname iname]
        if {$array_access} {
            set paramName $aname
        }

        ## get global basic object
        set basicObject [find objects ::$paramObject -isa basic]
        ## check for object
        if {$basicObject == {}} {
            ## :-) gimmick :-) find possible values that might fit
            set didYouMean [getDidYouMeanString ::$paramObject [find objects ::* -isa basic]]
            error "Unknown object in '$param': $didYouMean"
        }
        ## check if parameter is empty
        if {$paramName == {}} {
            error "Missing parameter for '$paramObject': Possible parameters are: [$basicObject getcompOption -remotecontrol yes]"
        }
        ## check for parameter in object
        if {([$basicObject getcomp $paramName] == {}) && !([$basicObject isa egse_ifdevice])} {
            ## :-) gimmick :-) find possible values that might fit
            set didYouMean [getDidYouMeanString $paramName [$basicObject getcomp]]
            error "Object '$paramObject': Unknown parameter '$paramName': $didYouMean"
        }

        ## check if array access is in valid range
        if {$array_access} {
            if {[$basicObject getcomp $paramName -array] == {}} {
                error "Parameter '$param': Is not an array"
            }
            if {[catch {$basicObject expandIndex $iname $paramName} msg]} {
                error "Parameter '$param': Indices out of range: '$param' array range [$basicObject getcomp $param -array]"
            }
        }

        ## with REPORT there are no values allowed
        if {$paramValue != {}} {
            error "Parameter '$param': No values allowed after parameter"
        }
    }

    ## no error
    return 1
}

